﻿local PLAYER,REALM,XREALM = 1,2,3
local FACTION,FACTIONLESS = true, false
local WELCOME, HELP, HELP_MAT, HELP_NOK, HELP_TOO_MANY = 1, 2, 3, 4, 5

local debugMode = true
local currPlayerIndex = 0
local isScanning = true
local isReplying = 0
local ORG_ChatFrame_MessageEventHandler = nil
local defaultItem = "Mooncloth"
local defaultReagent = "Felcloth"
local professionsTitle = ""
local selectedDDMIndex = 0

CraftBot_On = true
CraftBot_Search_Mode = REALM
CraftBot_Search_Mode_Faction = FACTION
CraftBot_Data = {}
CraftBot_Limit = 10
CraftBot_Silent = true
CraftBot_RefineReagents = false
CraftBot_Messages = {
	{"CRAFTBOT_WELCOME",CRAFTBOT_WELCOME},
	{"CRAFTBOT_HELP",CRAFTBOT_HELP},
	{"CRAFTBOT_HELP_MAT",CRAFTBOT_HELP_MAT},
	{"CRAFTBOT_HELP_NOK",CRAFTBOT_HELP_NOK},
	{"CRAFTBOT_HELP_TOO_MANY",CRAFTBOT_HELP_TOO_MANY},
}
CraftBot_IgnoredProfessions = {}

function CraftBot_IsIgnoredProfession(x)
	local profession = strupper(x)
	for i=1,#(CraftBot_IgnoredProfessions) do
		if CraftBot_IgnoredProfessions[i] == profession then
			return true
		end
	end
	return false
end

function CraftBot_SetIgnoredProfessions(x)
	if x and strtrim(x) ~= "" then
		x = strupper(strtrim(x))
		CraftBot_IgnoredProfessions = {}
		local delimiter = ","
		if strfind(x,delimiter) then
			local tmp = CraftBot_Split(x,delimiter)
			for i=1,#(tmp) do
				CraftBot_AddIgnoredProfession(tmp[i])
			end
		else
			CraftBot_AddIgnoredProfession(x)
		end
	end
end

function CraftBot_GetIgnoredProfessions()
	local x = ""
	for i=1,#(CraftBot_IgnoredProfessions) do
		if i == 1 then
			x = CraftBot_IgnoredProfessions[i]
		else
			x = x..","..CraftBot_IgnoredProfessions[i]
		end
	end
	CraftBotOptionFrameIgnoredProfessions:SetText(x,2)
end

function CraftBot_AddIgnoredProfession(x)
	if x and strtrim(x) ~= "" then
		local profession = strtrim(x)
		for i=1,#(CraftBot_IgnoredProfessions) do
			if CraftBot_IgnoredProfessions[i] == profession then
				return
			end
		end
		tinsert(CraftBot_IgnoredProfessions, profession)
		CraftBot_Print(format(CRAFTBOT_IGNOREPROFESSIONS, profession))
		for i=1,#(CraftBot_Data) do
			local skillIndex = 0
			if #(CraftBot_Data[i].skills) then
				for y=1,#(CraftBot_Data[i].skills) do
					if strupper(CraftBot_Data[i].skills[y].name) == profession then
						CraftBot_Data[i].skills[y].data = {}
						break
					end
				end
			end
		end
	end
end


function CraftBot_OnLoad()
	CraftBot:RegisterEvent("ADDON_LOADED")
	CraftBot:RegisterEvent("PLAYER_LOGIN")
	ORG_ChatFrame_MessageEventHandler = ChatFrame_MessageEventHandler
	ChatFrame_MessageEventHandler = CraftBot_ChatFrame_MessageEventHandler
end

function CraftBot_OnEvent(frame, event, arg_1, arg_2)
	if event == "CHAT_MSG_WHISPER" then
		CraftBot_OnWhisper(arg_1, arg_2)
	elseif event == "TRADE_SKILL_UPDATE" then
		CraftBot_Scan()
	elseif event == "ADDON_LOADED" and arg_1 == "CraftBot" then
		SlashCmdList["CRAFTBOT"] = CraftBot_SlashHandler
		SLASH_CRAFTBOT1 = "/craftbot"
		SLASH_CRAFTBOT2 = "/cb"
		CraftBot_Init()
		CraftBot_Activate(CraftBot_On)
	elseif event == "PLAYER_LOGIN" then
		isScanning = false
	end
end

function CraftBot_SlashHandler(msg)
	local words = {}
	for word in string.gmatch(msg, "%w+") do
		table.insert(words, word)
	end
	if #(words) > 0 then
		local cmd = words[1]
		if cmd == "on" then
			CraftBot_Activate(true)
		elseif cmd == "off" then
			CraftBot_Activate(false)
		elseif cmd == "send" or cmd == "s" then
			if #(words) > 2 then
				local user = words[2]
				if strupper(user) == "WIM" then
					user = CraftBot_GetWIMUser()
					if user == nil then
						user = words[2]
					end
				end
				local craftIndex = CraftBot_Strip(msg,cmd.." "..words[2].." ")
				if craftIndex ~= nil and craftIndex ~= "" then
					CraftBot_SendReagents(craftIndex, user)
				end
			end
		elseif cmd == "sendr" or cmd == "sr" then
			if #(words) > 2 then
				local user = words[2]
				if strupper(user) == "WIM" or strupper(user) == "W" then
					user = CraftBot_GetWIMUser()
					if user == nil then
						user = words[2]
					end
				end
				local craftIndex = CraftBot_Strip(msg,cmd.." "..words[2].." ")
				if craftIndex ~= nil and craftIndex ~= "" then
					CraftBot_SendRecipe(craftIndex, user)
				end
			end
		elseif cmd == "faction" or cmd == "f" then
			local param = words[2]
			if param == "on" then
				CraftBot_Search_Mode_Change("FACTION")
			elseif param == "off" then
				CraftBot_Search_Mode_Change("FACTIONLESS")
			end
		elseif cmd == "refine" or cmd == "r" then
			local param = words[2]
			if param == "on" then
				CraftBot_Refine_Mode_Change(true)
			elseif param == "off" then
				CraftBot_Refine_Mode_Change(false)
			end
		elseif cmd == "limit" or cmd == "l" then
			if words[2] and tonumber(words[2]) then
				CraftBot_Set_Limit(tonumber(words[2]))
			end
		elseif cmd == "search" then
			if words[2] then
				CraftBot_Search_Mode_Change(strupper(words[2]))
			end
		elseif cmd == "option" or cmd == "o" then
			CraftBot_ToggleOptions()
		elseif cmd == "silent" then
			local param = strlower(words[2])
			if param == "on" then
				CraftBot_Silent_Mode_Change(true)
			elseif param == "off" then
				CraftBot_Silent_Mode_Change(false)
			end
		elseif cmd == "reset" then
			CraftBot_Reset(1)
		elseif cmd == "resetall" then
			CraftBot_Reset(2)
		elseif cmd == "resetmsg" then
			CraftBot_Reset(3)
		end
	else
		CraftBot_Activate(not CraftBot_On)
	end
end

function CraftBot_ToggleOptions()
	if CraftBotOptionFrame:IsVisible() then
		CraftBotOptionFrame:Hide()
	else
		CraftBotOptionFrame:Show()
	end
end

function CraftBot_ChatFrame_MessageEventHandler(event)
	if CraftBot_On and CraftBot_Silent then
		if event == "CHAT_MSG_WHISPER_INFORM" then
			if isReplying > 0 then
				isReplying = isReplying - 1
				return false
			end
		elseif event == "CHAT_MSG_WHISPER" then
			if CraftBot_Strip(arg1,CRAFTBOT_WHISPERCMD) then
				return false
			end
		end
	end
	ORG_ChatFrame_MessageEventHandler(event)
	return true
end

function CraftBot_Set_Limit(x)
	if x >= 0 and x ~= CraftBot_Limit then
		CraftBot_Limit = x
		CraftBot_Print(format(CRAFTBOT_LIMIT, CraftBot_Limit))
	else
		CraftBot_Limit = 10
		CraftBot_Print(format(CRAFTBOT_LIMIT, CraftBot_Limit))
	end
end

function CraftBot_Reset(x)
	if x == 1 then
		CraftBot_Data[currPlayerIndex].skills = {}
		CraftBot_Print(CRAFTBOT_RESET)
	elseif x == 2 then
		CraftBot_Data = {}
		CraftBot_Init()
		CraftBot_Print(CRAFTBOT_RESET_ALL)
	elseif x == 3 then
		CraftBot_Messages = {
			{"CRAFTBOT_WELCOME",CRAFTBOT_WELCOME},
			{"CRAFTBOT_HELP",CRAFTBOT_HELP},
			{"CRAFTBOT_HELP_MAT",CRAFTBOT_HELP_MAT},
			{"CRAFTBOT_HELP_NOK",CRAFTBOT_HELP_NOK},
			{"CRAFTBOT_HELP_TOO_MANY",CRAFTBOT_HELP_TOO_MANY},
		}
		CraftBot_Print(CRAFTBOT_RESET_MSG)
	end
end

function CraftBot_Refine_Mode_Change(mode)
	if mode ~= CraftBot_RefineReagents then
		if mode then
			CraftBot_RefineReagents = true
			CraftBot_Print(CRAFTBOT_SMODE_REFINE_LABEL .. CRAFTBOT_SMODE_ON)
		else
			CraftBot_RefineReagents = false
			CraftBot_Print(CRAFTBOT_SMODE_REFINE_LABEL .. CRAFTBOT_SMODE_OFF)
		end
	end
end

function CraftBot_Silent_Mode_Change(mode)
	if mode ~= CraftBot_Silent then
		if mode then
			CraftBot_Silent = true
			CraftBot_Print(CRAFTBOT_SMODE_SILENT_LABEL .. CRAFTBOT_SMODE_ON)
		else
			CraftBot_Silent = false
			CraftBot_Print(CRAFTBOT_SMODE_SILENT_LABEL .. CRAFTBOT_SMODE_OFF)
		end
		if IsAddOnLoaded("CraftBotFu") then
			CraftBotFu:Update()
		end
	end
end

function CraftBot_Search_Mode_Change(mode)
	if (mode == "PLAYER" or mode == "P") and CraftBot_Search_Mode ~= PLAYER then
		CraftBot_Search_Mode = PLAYER
		CraftBot_Print(CRAFTBOT_SMODE_LABEL .. CRAFTBOT_SMODE_PLAYER)
	elseif (mode == "REALM" or mode == "R") and CraftBot_Search_Mode ~= REALM then
		CraftBot_Search_Mode = REALM
		CraftBot_Print(CRAFTBOT_SMODE_LABEL .. CRAFTBOT_SMODE_REALM)
	elseif (mode == "XREALM" or mode == "XR") and CraftBot_Search_Mode ~= XREALM then
		CraftBot_Search_Mode = XREALM
		CraftBot_Print(CRAFTBOT_SMODE_LABEL .. CRAFTBOT_SMODE_XREALM)
	elseif mode == "FACTION" and CraftBot_Search_Mode_Faction ~= FACTION then
		CraftBot_Search_Mode_Faction = FACTION
		CraftBot_Print(CRAFTBOT_SMODE_FACTION_LABEL .. CRAFTBOT_SMODE_ON)
	elseif mode == "FACTIONLESS" and CraftBot_Search_Mode_Faction ~= FACTIONLESS then
		CraftBot_Search_Mode_Faction = FACTIONLESS
		CraftBot_Print(CRAFTBOT_SMODE_FACTION_LABEL .. CRAFTBOT_SMODE_OFF)
	end
	if IsAddOnLoaded("CraftBotFu") then
		CraftBotFu:Update()
	end
end

function CraftBot_Scan()
	local skillName, currentLevel, maxLevel = GetTradeSkillLine()
	if skillName == nil or skillName == "" or skillName == "UNKNOWN" or CraftBot_IsIgnoredProfession(skillName) then
		return
	end
	local skillIndex = 0
	if #(CraftBot_Data[currPlayerIndex].skills) then
		for i=1, #(CraftBot_Data[currPlayerIndex].skills) do
			if CraftBot_Data[currPlayerIndex].skills[i].name == skillName then
				skillIndex = i
				CraftBot_Data[currPlayerIndex].skills[i].level = currentLevel
				break
			end
		end
	end
	if skillIndex == 0 then
		local skillNew = {
			["name"] = skillName,
			["specialization"] = "",
			["level"] = currentLevel,
			["data"] = {}
		}
		table.insert(CraftBot_Data[currPlayerIndex].skills, skillNew)
		skillIndex = #(CraftBot_Data[currPlayerIndex].skills)
	end
	local category = ""
	local categories = #(CraftBot_Split(BuildListString(GetTradeSkillSubClasses()), ",")) or 0
	local craftNum = GetNumTradeSkills() or 0
	if (craftNum - categories) <= #(CraftBot_Data[currPlayerIndex].skills[skillIndex].data) or isScanning then
		return
	end
	isScanning = true
	CraftBot_Data[currPlayerIndex].skills[skillIndex].data = {}
	for i=1, craftNum do
		if GetTradeSkillItemLink(i) then
			local craftData = {
		 		["craftName"] = "",
		 		["craftSubSpellName"] = "",
		 		["craftLink"] = "",
		 		["craftDesc"] = "",
		 		["qty"] = {
					["min"] = 0,
					["max"] = 0
				},
		 		["reagents"] = {},
		 		["category"] = ""
			}
			local craftName, skillType, numAvailable, isExpanded = GetTradeSkillInfo(i)
			craftData.craftName = craftName
			craftData.craftLink = GetTradeSkillItemLink(i)
			local itemstats = BuildListString(GetTradeSkillItemStats(i))
			-- adapted by Styler
			-- scanning tooltip for items that did not return a usable ItemDescription
			-- by calling GetTradeSkillItemStats(). This is mainly needes for GEM's
			if itemstats == nil then
				itemstats = ""
			end
			if itemstats == "" or not strfind(itemstats, ",", 1, true) then
				local desc = CraftBot_GetItemDescriptionFromTooltip(craftData.craftLink)
				if (desc ~= "") then
					itemstats = itemstats .. ", " .. desc
				end
			end
			craftData.craftDesc = itemstats
			craftData.category = category
			craftData.qty.min, craftData.qty.max = GetTradeSkillNumMade(i)
			for m=1,GetTradeSkillNumReagents(i) do
				local _, _, reagentCount, _ = GetTradeSkillReagentInfo(i, m)
				local reagentLink = GetTradeSkillReagentItemLink(i,m)
				if (reagentLink) then
					table.insert(craftData.reagents, { reagentLink, reagentCount })
				else
					isScanning = false
					CraftBot_Print(CRAFTBOT_SCAN_COMPLETE_ERROR)
					return
				end
			end
			table.insert(CraftBot_Data[currPlayerIndex].skills[skillIndex].data, craftData)
		else
			category, _, _, _ = GetTradeSkillInfo(i)
			if not category then
				category = ""
			end
		end
	end
	isScanning = false
	CraftBot_Print(CRAFTBOT_SCAN_COMPLETE)
	CraftBot_SetShoppeProfessions()
	if IsAddOnLoaded("CraftBotFu") then
		CraftBotFu:Update()
	end
end

function CraftBot_Activate(state)
	if (state) then
		CraftBot:RegisterEvent("CHAT_MSG_WHISPER")
		CraftBot:RegisterEvent("TRADE_SKILL_UPDATE")
		CraftBot_On = true
		CraftBot_Print(CRAFTBOT_ON)
		CraftBot_SetShoppeProfessions()
	else
		CraftBot:UnregisterEvent("CHAT_MSG_WHISPER")
		CraftBot:UnregisterEvent("TRADE_SKILL_UPDATE")
		CraftBot_On = false
		CraftBot_Print(CRAFTBOT_OFF)
	end
	if IsAddOnLoaded("CraftBotFu") then
		CraftBotFu:Update()
	end
end

function CraftBot_Init()
	local name = UnitName("player")
	local realm = GetRealmName()
	local _, localFac = UnitFactionGroup("player")
	if #(CraftBot_Data) > 0 then
		for i=1, #(CraftBot_Data) do
			if CraftBot_Data[i].name == name and CraftBot_Data[i].realm == realm then
				currPlayerIndex = i
				break
			end
		end
	end
	if currPlayerIndex == 0 then
		local playerData = {
			["name"] = name,
			["realm"] = realm,
			["faction"] = localFac,
			["skills"] = {}
		}
		table.insert(CraftBot_Data, playerData)
		currPlayerIndex = #(CraftBot_Data)
	else
		if CraftBot_Data[currPlayerIndex].faction == nil then
			CraftBot_Data[currPlayerIndex].faction = localFac
		end
	end
end

function CraftBot_SetShoppeProfessions()
	local setDefaults = false
	for i=1, #(CraftBot_Data) do
		if CraftBot_Searchable(i) then
			for x=1, #(CraftBot_Data[i].skills) do
				if strfind(professionsTitle, CraftBot_Data[i].skills[x].name, 1, true) then
				else
					if x ~= #(CraftBot_Data[i].skills) then
						professionsTitle = professionsTitle .. CraftBot_Data[i].skills[x].name .. "/"
					else
						professionsTitle = professionsTitle .. CraftBot_Data[i].skills[x].name
					end
				end
				if not setDefaults then
					for y=#(CraftBot_Data[i].skills[x].data), 1, -1 do
						local craftName = CraftBot_Data[i].skills[x].data[y].craftName
						if craftName ~= nil and #(CraftBot_Data[i].skills[x].data[y].reagents) > 0 then
							local reagentName, _, _, _, _, _, _, _, _, _ = GetItemInfo(CraftBot_Data[i].skills[x].data[y].reagents[1][1])
							if reagentName ~= nil then
								if (strlen(CraftBot_Messages[HELP][2]) + strlen(craftName) + strlen(reagentName) - 4) <= 255 then
									defaultItem = craftName
									defaultReagent = reagentName
									setDefaults = true
								end
							end
						end
					end
				end
			end
			if CraftBot_Search_Mode == PLAYER then
				return
			end
		end
	end
end

function CraftBot_SendChat(user, text)
	if strlower(user) == strlower(UnitName("player")) then
		CraftBot_Print(text)
	else
		if CraftBot_Silent then 
			isReplying = isReplying + 1
		end
		SendChatMessage(text, "WHISPER", GetDefaultLanguage("player"), user)
	end
end

function CraftBot_CheckHolidays(user)
	local day, month = tonumber(date("%d")), tonumber(date("%m"))
	for i=1, #(CraftBot_Holidays) do
		for x = 1, #(CraftBot_Holidays[i].dates) do
			if month == CraftBot_Holidays[i].dates[x][1] and day == CraftBot_Holidays[i].dates[x][2] then
				CraftBot_SendChat(user, CraftBot_Holidays[i].greetings)
				return
			end
		end
	end
end

function CraftBot_OnWhisper(text, user)
	text = strtrim(text)
	if text == CRAFTBOT_WHISPERCMD_HELP or text == CRAFTBOT_WHISPERCMD_SEARCH then
		CraftBot_CheckHolidays(user)
		CraftBot_SendChat(user,format(CraftBot_Messages[WELCOME][2],UnitName("player"),professionsTitle))
		CraftBot_SendChat(user,format(CraftBot_Messages[HELP][2],defaultItem,defaultReagent))
		return
	end
	local mode = 0
	local tmp = CraftBot_Strip(text,CRAFTBOT_WHISPERCMD_SEARCH.." ")
	if tmp == nil then
		tmp = CraftBot_Strip(text,CRAFTBOT_WHISPERCMD_USE.." ")
		if tmp == nil then
			tmp = CraftBot_Strip(text,CRAFTBOT_WHISPERCMD_REAGENTS.." ")
			if tmp == nil then
				return
			end
		else
			mode = 2
		end
	else
		mode = 1
	end
	if tmp == "" then
		return
	else
		if mode == 1 then
			CraftBot_Search(tmp, user)
		elseif mode == 2 then
			CraftBot_SendRecipe(tmp, user)
		else
			CraftBot_SendReagents(tmp, user)
		end
	end
end

function CraftBot_Search(msg, user)
	local orgMsg = msg
	local found = {}
	msg = strlower(msg)
	local CountIt = 0
	local exceedLimit = false
	for i=1, #(CraftBot_Data) do
		if CraftBot_Searchable(i) then
			for x=1, #(CraftBot_Data[i].skills) do
				for y=1, #(CraftBot_Data[i].skills[x].data) do
					local craftData = CraftBot_Data[i].skills[x].data[y]
					local craftName, craftLink, craftDesc, craftMin, craftMax, craftCategory = craftData.craftName, craftData.craftLink, craftData.craftDesc, craftData.qty.min, craftData.qty.max, craftData.category
					local foo = nil
					if foo == nil and strfind(orgMsg,"|Hitem:",1,true) then
						-- Adapted from Styler
						if CraftBot_GetItemID(craftLink) == CraftBot_GetItemID(orgMsg) then
							foo = true
						end
					end
					if foo == nil then
						-- So that multikeywords can span both craft names and descriptions
						local craftText = craftName
						if craftDesc ~= nil and craftDesc ~= "" then
							craftText = craftText .. ", " .. craftDesc
						end
						foo = CraftBot_MatchStrings(strlower(craftText),msg,",")
					end
					if foo == nil and craftCategory ~= nil and craftCategory ~= "" then
						foo = CraftBot_MatchStrings(strlower(craftCategory),msg,",")
					end
					if foo ~= nil then
						local duplicate = false
						for z=1, #(found) do
							if found[z] == craftName then
								duplicate = true
								break
							end
						end
						if not duplicate then
							CountIt = CountIt + 1
							if CraftBot_Limit > 0 and CountIt > CraftBot_Limit then
								exceedLimit = true
								break
							end
							if craftMin == 1 and craftMax == 1 then
								CraftBot_SendChat(user, format(CRAFTBOT_RESULT,i,x,y,craftLink))
							else
								if craftMin == craftMax then
									CraftBot_SendChat(user, format(CRAFTBOT_RESULT_QTY,i,x,y,craftLink,craftMin))
								else
									CraftBot_SendChat(user, format(CRAFTBOT_RESULT_QTY2,i,x,y,craftLink,craftMin,craftMax))
								end
							end
							tinsert(found,craftName)
						end
					end
				end
				if exceedLimit then
					break
				end
			end
			if CraftBot_Search_Mode == PLAYER or exceedLimit then
				break
			end
		end
	end
	if CountIt > 0 then
		if exceedLimit then
			CountIt = CraftBot_Limit
			CraftBot_SendChat(user,format(CraftBot_Messages[HELP_TOO_MANY][2],CraftBot_Limit))
		end
		CraftBot_SendChat(user,CraftBot_Messages[HELP_MAT][2])
		CraftBot_Print(format(CRAFTBOT_RESULTS_OK,orgMsg,user,user,CountIt))
	else
		CraftBot_SendChat(user, CraftBot_Messages[HELP_NOK][2])
		CraftBot_Print(format(CRAFTBOT_RESULTS_NOK,orgMsg,user,user))
	end
end

function CraftBot_SendReagents(msg, user)
	if #(CraftBot_Data) == 0 then
		return
	end
	local playerIndex,skillIndex,craftIndex = 0,0,0
	local isDotFormat = false
	-- playerIndex.skillIndex.craftIndex format
	if strfind(msg,".") then
		local tmp = CraftBot_Split(msg,".")
		if #(tmp) ~= 3 then
		else
			playerIndex,skillIndex,craftIndex = tonumber(tmp[1]),tonumber(tmp[2]),tonumber(tmp[3])
			if playerIndex ~= nil and skillIndex ~= nil and craftIndex ~= nil and CraftBot_Data[playerIndex] ~= nil and CraftBot_Data[playerIndex].skills[skillIndex] ~= nil and CraftBot_Data[playerIndex].skills[skillIndex].data[craftIndex] ~= nil then
				isDotFormat = true
			end
		end
	end
	if not isDotFormat then
		isDotFormat,playerIndex,skillIndex,craftIndex = CraftBot_GetDotFormat(msg)
	end
	if isDotFormat then
		local reagentsData = {}
		if CraftBot_RefineReagents then
			reagentsData = CraftBot_GetAllReagents(CraftBot_Data[playerIndex].skills[skillIndex].data[craftIndex].reagents)
		else
			reagentsData = CraftBot_Data[playerIndex].skills[skillIndex].data[craftIndex].reagents
		end
		local reagents = " "
		local rtmp = nil
		local craftLink = CraftBot_Data[playerIndex].skills[skillIndex].data[craftIndex].craftLink
		CraftBot_SendChat(user, format(CRAFTBOT_MAT_LIST,craftLink))
		for m=1, #(reagentsData) do
			local reagentLink, reagentCount = reagentsData[m][1], reagentsData[m][2]
			rtmp = reagents.." "..reagentCount.." x "..reagentLink
			if strlen(rtmp) > 255 then
				CraftBot_SendChat(user, reagents)
				rtmp = " " .. reagentCount.." x "..reagentLink
				reagents = rtmp
			end
			reagents = rtmp
		end
		CraftBot_SendChat(user, reagents)
		CraftBot_Print(format(CRAFTBOT_MAT_OK,craftLink,user,user))
	else
		CraftBot_Print(format(CRAFTBOT_MAT_NOK,msg,user,user))
	end
end

function CraftBot_GetDotFormat(msg)
	if strfind(msg, "|Hitem:") then
		for i=1, #(CraftBot_Data) do
			if CraftBot_Searchable(i) then
				for x=1, #(CraftBot_Data[i].skills) do
					for y=1, #(CraftBot_Data[i].skills[x].data) do
						if CraftBot_GetItemID(CraftBot_Data[i].skills[x].data[y].craftLink) == CraftBot_GetItemID(msg) then
							return true,i,x,y
						end
					end
				end
				if CraftBot_Search_Mode == PLAYER then
					break
				end
			end
		end
	else
		for i=1, #(CraftBot_Data) do
			if CraftBot_Searchable(i) then
				for x=1, #(CraftBot_Data[i].skills) do
					for y=1, #(CraftBot_Data[i].skills[x].data) do
						local craftText, craftDesc = CraftBot_Data[i].skills[x].data[y].craftName, CraftBot_Data[i].skills[x].data[y].craftDesc
						if craftDesc ~= nil and craftDesc ~= "" then
							craftText = craftText .. ", " .. craftDesc
						end
						if CraftBot_MatchStrings(strlower(craftText),msg,",") then
							return true,i,x,y
						end
					end
				end
				if CraftBot_Search_Mode == PLAYER then
					break
				end
			end
		end
	end
	return false
end

function CraftBot_Searchable(i)
	local _, localFac = UnitFactionGroup("player")
	if CraftBot_Search_Mode == REALM and CraftBot_Data[i].realm == GetRealmName() then
		if not CraftBot_Search_Mode_Faction or (CraftBot_Search_Mode_Faction and CraftBot_Data[i].faction == localFac) then
			return true
		end
	elseif CraftBot_Search_Mode == PLAYER and i == currPlayerIndex then
		return true
	elseif CraftBot_Search_Mode == XREALM then
		if not CraftBot_Search_Mode_Faction or (CraftBot_Search_Mode_Faction and CraftBot_Data[i].faction == localFac) then
			return true
		end
	end
	return false
end

function CraftBot_SendRecipe(reagent, user)
	local orgreagent = reagent
	reagent = strlower(reagent)
	local itemName = nil
	if strfind(reagent,"|hitem:") then
		itemName, _, _, _, _, _, _, _, _, _ = GetItemInfo(reagent)
	else
		itemName = reagent
	end
	if itemName ~= nil then
		local CountIt = 0
		local exceedLimit = false
		for i=1, #(CraftBot_Data) do
			if CraftBot_Searchable(i) then
				for x=1, #(CraftBot_Data[i].skills) do
					for y=1, #(CraftBot_Data[i].skills[x].data) do
						local reagentsData = CraftBot_Data[i].skills[x].data[y].reagents
						for m=1, #(reagentsData) do
							local reagentLink = reagentsData[m][1]
							-- Adapted from Styler
							if strfind(strlower(reagentLink),reagent,1,true) or CraftBot_GetItemID(reagentLink) == CraftBot_GetItemID(orgreagent) then
								CountIt = CountIt + 1
								if CraftBot_Limit > 0 and CountIt > CraftBot_Limit then
									exceedLimit = true
									break
								end
								CraftBot_SendChat(user, format(CRAFTBOT_RESULT,i,x,y,CraftBot_Data[i].skills[x].data[y].craftLink))
								break
							end
						end
						if exceedLimit then
							break
						end
					end
					if exceedLimit then
						break
					end
				end
				if CraftBot_Search_Mode == PLAYER or exceedLimit then
					break
				end
			end
		end
		if CountIt > 0 then
			if exceedLimit then
				CountIt = CraftBot_Limit
				CraftBot_SendChat(user,format(CraftBot_Messages[HELP_TOO_MANY][2],CraftBot_Limit))
			end
			CraftBot_SendChat(user,CraftBot_Messages[HELP_MAT][2])
			CraftBot_Print(format(CRAFTBOT_RESULTS2_OK,orgreagent,user,user,CountIt))
			return
		end
		CraftBot_Print(format(CRAFTBOT_RESULTS2_NOK,orgreagent,user,user))
		return
	end
	CraftBot_Print(format(CRAFTBOT_RESULTS2_NOK,"nil",user,user))
	return
end

-- Functions for Titan and FuBar plugins
function CraftBot_GetTooltipText()
	local text = ""
	for i=1, #(CraftBot_Data) do
		if CraftBot_Searchable(i) then
			local craftNum = #(CraftBot_Data[i].skills)
			if craftNum > 0 then
				if text ~= "" then
					text = text.."\n"
				end
				text = text.."|cffffffff"..CraftBot_Data[i].name.."|r\n"
				for x=1, craftNum do
					text = text..format(CRAFTBOT_TOOLTIP_SKILL,CraftBot_Data[i].skills[x].name,#(CraftBot_Data[i].skills[x].data))
					if x ~= craftNum then
						text = text.."\n"
					end
				end
			end
			if CraftBot_Search_Mode == PLAYER then
				break
			end
		end
	end
	return text
end

function CraftBot_GetButtonText()
	if (CraftBot_On == false) then
		return CRAFTBOT_SMODE_OFF
	else
		if CraftBot_Search_Mode == PLAYER then
			return CRAFTBOT_SMODE_PLAYER
		elseif CraftBot_Search_Mode == REALM then
			return CRAFTBOT_SMODE_REALM
		elseif CraftBot_Search_Mode == XREALM then
			return CRAFTBOT_SMODE_XREALM
		end
	end
	return ""
end

-- Support Functions
function CraftBot_GetWIMUser()
	if IsAddOnLoaded("WIM") then
		if WIM_EditBoxInFocus then
			local found, _, user = strfind(WIM_EditBoxInFocus:GetName(), "^WIM_msgFrame(.+)MsgBox")
			if found then
				return user
			end
		end
	end
	return nil
end

function CraftBot_Split(text, separator) 
	local value
   	local x, y = 1
	local t = {}
	local regexpstring = "([^"..separator.."]+)"
	if (text and separator) then
		x, y, value = strfind(text, regexpstring, x)
	   	while (value) do
			table.insert(t, value)
			x = y + 1
			x, y, value = strfind(text, regexpstring, x)
	   	end
	end
	return t
end

function CraftBot_Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg)
end

function CraftBot_Debug(msg)
	if debugMode then
		DEFAULT_CHAT_FRAME:AddMessage(msg)
	end
end

function CraftBot_SetOptionValue(button)
	local name = button:GetParent():GetName()
	local checked = false
	if button:GetChecked() then
		checked = true
	end
	if name == "CraftBotOptionFrameSearchXR" and checked then
		CraftBot_Search_Mode_Change("XR")
		getglobal("CraftBotOptionFrameSearchR_Button"):SetChecked(false)
		getglobal("CraftBotOptionFrameSearchP_Button"):SetChecked(false)
	elseif name == "CraftBotOptionFrameSearchR" and checked then
		CraftBot_Search_Mode_Change("R")
		getglobal("CraftBotOptionFrameSearchXR_Button"):SetChecked(false)
		getglobal("CraftBotOptionFrameSearchP_Button"):SetChecked(false)
	elseif name == "CraftBotOptionFrameSearchP" and checked then
		CraftBot_Search_Mode_Change("P")
		getglobal("CraftBotOptionFrameSearchXR_Button"):SetChecked(false)
		getglobal("CraftBotOptionFrameSearchR_Button"):SetChecked(false)
	elseif name == "CraftBotOptionFrameFaction" then
		if checked then
			CraftBot_Search_Mode_Change("FACTION")
		else
			CraftBot_Search_Mode_Change("FACTIONLESS")
		end
	elseif name == "CraftBotOptionFrameSilent" then
		if checked then
			CraftBot_Silent_Mode_Change(true)
		else
			CraftBot_Silent_Mode_Change(false)
		end
	elseif name == "CraftBotOptionFrameRefine" then
		if checked then
			CraftBot_Refine_Mode_Change(true)
		else
			CraftBot_Refine_Mode_Change(false)
		end
	end
end

function CraftBot_GetOptionValue(button)
	local name = button:GetName()
	local fontString = getglobal(button:GetName().."_Text")
	local radioButton = getglobal(button:GetName().."_Button")
	if name == "CraftBotOptionFrameSearchXR" then
		if CraftBot_Search_Mode == XREALM then
			radioButton:SetChecked(true)
		else
			radioButton:SetChecked(false)
		end
		fontString:SetText(CRAFTBOT_SMODE_XREALM)
	elseif name == "CraftBotOptionFrameSearchR" then
		if CraftBot_Search_Mode == REALM then
			radioButton:SetChecked(true)
		else
			radioButton:SetChecked(false)
		end
		fontString:SetText(CRAFTBOT_SMODE_REALM)
	elseif name == "CraftBotOptionFrameSearchP" then
		if CraftBot_Search_Mode == PLAYER then
			radioButton:SetChecked(true)
		else
			radioButton:SetChecked(false)
		end
		fontString:SetText(CRAFTBOT_SMODE_PLAYER)
	elseif name == "CraftBotOptionFrameFaction" then
		if CraftBot_Search_Mode_Faction == FACTION then
			radioButton:SetChecked(true)
		else
			radioButton:SetChecked(false)
		end
		fontString:SetText(CRAFTBOT_SMODE_FACTION)
	elseif name == "CraftBotOptionFrameSilent" then
		if CraftBot_Silent then
			radioButton:SetChecked(true)
		else
			radioButton:SetChecked(false)
		end
		fontString:SetText(CRAFTBOT_SMODE_SILENT)
	elseif name == "CraftBotOptionFrameRefine" then
		if CraftBot_RefineReagents then
			radioButton:SetChecked(true)
		else
			radioButton:SetChecked(false)
		end
		fontString:SetText(CRAFTBOT_SMODE_REFINE)
	end
end

function CraftBot_Localization_DDM_OnLoad()
	UIDropDownMenu_Initialize(this:GetParent(),CraftBot_Localization_DDM_Init)
	UIDropDownMenu_SetAnchor(0,0,this:GetParent(),"TOPRIGHT",this:GetName(),"BOTTOMRIGHT")
end

function CraftBot_Localization_DDM_Init()
	local info = {}
	for i=1,#(CraftBot_Messages) do
		info = {
			text = CraftBot_Messages[i][1],
			value = i,
			func = CraftBot_Localization_DDM_Select
		}
		if i == selectedDDMIndex then 
			info.checked = 1
		end
		UIDropDownMenu_AddButton(info)
	end
end

function CraftBot_Localization_DDM_Select()
	if this.value then
		local text = strtrim(CraftBotOptionFrameLocalization:GetText())
		selectedDDMIndex = this.value
		CraftBotOptionFrameLocalization:SetText(CraftBot_Messages[this.value][2])
	end
end

function CraftBot_SetLocalization(text)
	local x = strtrim(text)
	if x ~= "" and selectedDDMIndex > 0 and CraftBot_Messages[selectedDDMIndex][2] ~= x then
		CraftBot_Messages[selectedDDMIndex][2] = x
		CraftBot_Print(CraftBot_Messages[selectedDDMIndex][1]..": "..x)
	end
end

function CraftBot_ShowCustomTooltip(x, y)
	if x then
		if y == "Localization" then
			GameTooltip:SetOwner(CraftBotOptionFrameLocalization, "ANCHOR_BOTTOMRIGHT",CraftBotOptionFrameLocalization:GetWidth()*-1, 0)
			GameTooltip:ClearLines()
			for i=1, #(CRAFTBOT_TOOLTIP_LOCALIZATION) do
				GameTooltip:AddLine(CRAFTBOT_TOOLTIP_LOCALIZATION[i][1], CRAFTBOT_TOOLTIP_LOCALIZATION[i][2], CRAFTBOT_TOOLTIP_LOCALIZATION[i][3], CRAFTBOT_TOOLTIP_LOCALIZATION[i][4])
			end
	 		GameTooltip:Show()
		elseif y == "Limit" then
			GameTooltip:SetOwner(CraftBotOptionFrameLimit, "ANCHOR_BOTTOMRIGHT",CraftBotOptionFrameLimit:GetWidth()*-1, 0)
			GameTooltip:ClearLines()
			for i=1, #(CRAFTBOT_TOOLTIP_LIMIT) do
				GameTooltip:AddLine(CRAFTBOT_TOOLTIP_LIMIT[i][1], CRAFTBOT_TOOLTIP_LIMIT[i][2], CRAFTBOT_TOOLTIP_LIMIT[i][3], CRAFTBOT_TOOLTIP_LIMIT[i][4])
			end
	 		GameTooltip:Show()
		elseif y == "IgnoredProfessions" then
			GameTooltip:SetOwner(CraftBotOptionFrameIgnoredProfessions, "ANCHOR_BOTTOMRIGHT",CraftBotOptionFrameIgnoredProfessions:GetWidth()*-1, 0)
			GameTooltip:ClearLines()
			for i=1, #(CRAFTBOT_TOOLTIP_IGNOREDPROFESSIONS) do
				GameTooltip:AddLine(CRAFTBOT_TOOLTIP_IGNOREDPROFESSIONS[i][1], CRAFTBOT_TOOLTIP_IGNOREDPROFESSIONS[i][2], CRAFTBOT_TOOLTIP_IGNOREDPROFESSIONS[i][3], CRAFTBOT_TOOLTIP_IGNOREDPROFESSIONS[i][4])
			end
	 		GameTooltip:Show()
		end
	else
		GameTooltip:Hide()
	end
end

function CraftBot_Strip(x, y)
	local p, n = strfind(strlower(strtrim(x)), strlower(y), 1, true)
	if p and p == 1 and n then
		return strtrim(strsub(x, n+1, strlen(x)))
	end
	return nil
end

-- Functions from Styler
-- returns the ItemID from an intemLink
function CraftBot_GetItemID(itemLink)
	if itemLink ~= nil and itemLink ~= "" then
		local found, _, itemString = strfind(itemLink, "^|%x+|H(.+)|h%[.+%]")
		if found ~= nil then
			local found, _, itemId = strfind(itemString, "^item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)")
			if found ~= nil then
				return itemId
			end
		end
	end
	return 0
end

function CraftBot_GetItemDescriptionFromTooltip(itemlink)
	local desc = {}
	local found, _, itemString = strfind(itemlink, "^|%x+|H(.+)|h%[.+%]")
	if (found) then
		CrafBotScanningTooltip:SetOwner(CraftBot, "ANCHOR_NONE")
		CrafBotScanningTooltip:ClearLines()
		CrafBotScanningTooltip:SetHyperlink(itemString)
		if (CrafBotScanningTooltip:NumLines() >= 3) then
			for l=2, CrafBotScanningTooltip:NumLines(), 1 do
				local tttext = getglobal("CrafBotScanningTooltipTextLeft"..l):GetText()
				if (tttext ~= "") then tinsert(desc, tttext) end
			end
		end
	end
	return table.concat(desc, ", ")
end

function CraftBot_MatchStrings(haystack, needle, sep)
	local found = true
	local needleArray = CraftBot_Split(needle, sep)
	if strfind(haystack, sep, 1, true) then
		local needleArray = CraftBot_Split(needle, sep)
		for i=1, #(needleArray), 1 do
			if not strfind(haystack, strtrim(needleArray[i]) ,1 , true) then
				return nil
			end
		end
	else
		found = strfind(haystack, needle, 1, true)
	end
	return found
end

function CraftBot_RefineScan()
	isScanning = true
	for i=1, #(CraftBot_Data) do
		for x=1,#(CraftBot_Data[i].skills) do
			for y=1,#(CraftBot_Data[i].skills[x].data) do
				CraftBot_Data[i].skills[x].data[y].reagents = CraftBot_GetAllReagents(CraftBot_Data[i].skills[x].data[y].reagents)
			end
		end
	end
	isScanning = false
end

function CraftBot_GetAllReagents(reagentData)
	if #(CraftBot_Data) then
		repeat
			local refine = false
			local newReagentData = {}
			for i=1,#(reagentData) do
				local tmpReagentData = CraftBot_GetReagentReagentData(reagentData[i][1])
				if #(tmpReagentData) > 0 then
					for j=1, #(tmpReagentData) do
						local link = tmpReagentData[j][1]
						local qty = tmpReagentData[j][2] * reagentData[i][2]
						tinsert(newReagentData, { link, qty })
					end
					refine = true
				else
					tinsert(newReagentData, reagentData[i])
				end
			end
			reagentData = newReagentData
		until not refine
	end
	return reagentData
end

function CraftBot_GetReagentReagentData(link)
	for i=1, #(CraftBot_Data) do
		if CraftBot_Searchable(i) then
			for x=1,#(CraftBot_Data[i].skills) do
				for y=1,#(CraftBot_Data[i].skills[x].data) do
					if CraftBot_Data[i].skills[x].data[y].craftLink == link then
						return CraftBot_Data[i].skills[x].data[y].reagents
					end
				end
			end
			if CraftBot_Search_Mode == PLAYER then
				break
			end
		end
	end
	return {}
end
